use core::fmt::Write;
use embedded_graphics::{
    geometry::{Point, Size},
    mono_font::{ascii::FONT_10X20, MonoTextStyleBuilder},
    pixelcolor::{Rgb565, RgbColor, WebColors},
    primitives::{Primitive, PrimitiveStyleBuilder, Rectangle},
    text::{Baseline, Text},
    Drawable,
};
use heapless::String;

use crate::voltage::VoltMeter;

use super::Element;

const WIDTH: u32 = 320;
const BORDER_WIDTH: u32 = 0;
const BACKGROUND_COLOR: Rgb565 = Rgb565::CSS_DARK_ORANGE;

pub struct StatusBar {
    volt_meter: VoltMeter,
}

impl StatusBar {
    pub fn new(volt_meter: VoltMeter) -> Self {
        Self { volt_meter }
    }
}

impl Element for StatusBar {
    fn render<
        E,
        DT: embedded_graphics::prelude::DrawTarget<
            Color = embedded_graphics::pixelcolor::Rgb565,
            Error = E,
        >,
    >(
        &mut self,
        dt: &mut DT,
    ) -> Result<(), E> {
        let (batt, usb) = self.volt_meter.voltage();

        let rect_style = PrimitiveStyleBuilder::new()
            .fill_color(BACKGROUND_COLOR)
            .build();
        Rectangle::new(Point::new(0, 0), Size::new(WIDTH, BORDER_WIDTH * 2 + 20))
            .into_styled(rect_style)
            .draw(dt)?;

        let text_style = MonoTextStyleBuilder::new()
            .font(&FONT_10X20)
            .text_color(Rgb565::WHITE)
            .build();

        let mut s: String<32> = String::new();
        if usb > 1.0 {
            write!(&mut s, "BAT: {batt:.2}V      USB: {usb:.2}V").unwrap();
        } else {
            write!(&mut s, "BAT: {batt:.2}V").unwrap();
        }

        Text::with_baseline(
            &s,
            Point::new(8, BORDER_WIDTH.try_into().unwrap()),
            text_style,
            Baseline::Top,
        )
        .draw(dt)?;

        Ok(())
    }

    fn key_push(&mut self, _: crate::keyboard::KeyCode) {}
}
